@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2023 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -36,6 +36,7 @@ import (
 
 type packCmd struct {
 	CheckSkeleton bool   `long:"check-skeleton"`
+	AppendVerity  bool   `long:"append-integrity-data" hidden:"yes"`
 	Filename      string `long:"filename"`
 	Compression   string `long:"compression"`
 	Positional    struct {
@@ -60,8 +61,13 @@ When used with --check-skeleton, pack only checks whether snap-dir contains
 valid snap metadata and raises an error otherwise. Application commands listed
 in snap metadata file, but appearing with incorrect permission bits result in an
 error. Commands that are missing from snap-dir are listed in diagnostic
-messages.
-`)
+messages.`,
+
+/*
+When used with --append-integrity-data, pack will append dm-verity data at the end
+of the snap to be used with snapd's snap integrity verification mechanism.
+*/
+)
 
 func init() {
 	cmd := addCommand("pack",
@@ -76,6 +82,8 @@ func init() {
 			"filename": i18n.G("Output to this filename"),
 			// TRANSLATORS: This should not start with a lowercase letter.
 			"compression": i18n.G("Compression to use (e.g. xz or lzo)"),
+			// TRANSLATORS: This should not start with a lowercase letter.
+			"append-integrity-data": i18n.G("Generate and append dm-verity data"),
 		}, nil)
 	cmd.extra = func(cmd *flags.Command) {
 		// TRANSLATORS: this describes the default filename for a snap, e.g. core_16-2.35.2_amd64.snap
@@ -111,6 +119,7 @@ func (x *packCmd) Execute([]string) error {
 		TargetDir:   x.Positional.TargetDir,
 		SnapName:    x.Filename,
 		Compression: x.Compression,
+		Integrity:   x.AppendVerity,
 	})
 	if err != nil {
 		// TRANSLATORS: the %q is the snap-dir (the first positional
