@@ -26,20 +26,20 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"runtime"
 	"strconv"
 	"strings"
 	"time"
 
 	. "gopkg.in/check.v1"
 
-	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/boot"
-	"github.com/snapcore/snapd/boot/boottest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/gadget/gadgettest"
 	"github.com/snapcore/snapd/gadget/install"
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
 	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/secboot/keys"
@@ -83,6 +83,13 @@ func (s *installSuite) TestInstallRunSimpleHappy(c *C) {
 	})
 }
 
+func (s *installSuite) TestInstallRunSimpleHappyFromMountPoint(c *C) {
+	s.testInstall(c, installOpts{
+		encryption: false,
+		fromSeed:   true,
+	})
+}
+
 func (s *installSuite) TestInstallRunEncryptedLUKS(c *C) {
 	s.testInstall(c, installOpts{
 		encryption: true,
@@ -106,6 +113,7 @@ func (s *installSuite) TestInstallRunEncryptionExistingPartitions(c *C) {
 type installOpts struct {
 	encryption    bool
 	existingParts bool
+	fromSeed      bool
 }
 
 func (s *installSuite) testInstall(c *C, opts installOpts) {
@@ -151,9 +159,28 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 	mockPartx := testutil.MockCommand(c, "partx", "")
 	defer mockPartx.Restore()
 
-	mockUdevadm := testutil.MockCommand(c, "udevadm", "")
+	mockUdevadm := testutil.MockCommand(c, "udevadm", `
+if [ "$*" = "info --query property --name /dev/mmcblk0p1" ]; then
+	echo "ID_PART_ENTRY_DISK=42:0"
+elif [ "$*" = "info --query property --name /dev/block/42:0" ]; then
+	echo "DEVNAME=/dev/mmcblk0"
+	echo "DEVPATH=/devices/virtual/mmcblk0"
+	echo "DEVTYPE=disk"
+	echo "ID_PART_TABLE_UUID=some-gpt-uuid"
+	echo "ID_PART_TABLE_TYPE=GPT"
+fi
+`)
 	defer mockUdevadm.Restore()
 
+	if opts.fromSeed {
+		restoreMountInfo := osutil.MockMountInfo(`130 30 42:1 / /run/mnt/ubuntu-seed rw,relatime shared:54 - vfat /dev/mmcblk0p1 rw
+`)
+		defer restoreMountInfo()
+	} else {
+		restoreMountInfo := osutil.MockMountInfo(``)
+		defer restoreMountInfo()
+	}
+
 	mockCryptsetup := testutil.MockCommand(c, "cryptsetup", "")
 	defer mockCryptsetup.Restore()
 
@@ -162,70 +189,9 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 		defer mockBlockdev.Restore()
 	}
 
-	restore = install.MockEnsureNodesExist(func(dss []gadget.OnDiskStructure, timeout time.Duration) error {
+	restore = install.MockEnsureNodesExist(func(nodes []string, timeout time.Duration) error {
 		c.Assert(timeout, Equals, 5*time.Second)
-		c.Assert(dss, DeepEquals, []gadget.OnDiskStructure{
-			{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-boot",
-						Label:      "ubuntu-boot",
-						Size:       750 * quantity.SizeMiB,
-						Type:       "0C",
-						Role:       gadget.SystemBoot,
-						Filesystem: "vfat",
-					},
-					StartOffset: (1 + 1200) * quantity.OffsetMiB,
-					YamlIndex:   1,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 2,
-				Node:      "/dev/mmcblk0p2",
-				Size:      750 * quantity.SizeMiB,
-			},
-			{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-save",
-						Label:      "ubuntu-save",
-						Size:       16 * quantity.SizeMiB,
-						Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-						Role:       gadget.SystemSave,
-						Filesystem: "ext4",
-					},
-					StartOffset: (1 + 1200 + 750) * quantity.OffsetMiB,
-					YamlIndex:   2,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 3,
-				Node:      "/dev/mmcblk0p3",
-				Size:      16 * quantity.SizeMiB,
-			},
-			{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-data",
-						Label:      "ubuntu-data",
-						// TODO: this is set from the yaml, not from the actual
-						// calculated disk size, probably should be updated
-						// somewhere
-						Size:       1500 * quantity.SizeMiB,
-						Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-						Role:       gadget.SystemData,
-						Filesystem: "ext4",
-					},
-					StartOffset: (1 + 1200 + 750 + 16) * quantity.OffsetMiB,
-					YamlIndex:   3,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 4,
-				Node:      "/dev/mmcblk0p4",
-				Size:      (30528 - (1 + 1200 + 750 + 16)) * quantity.SizeMiB,
-			},
-		})
+		c.Assert(nodes, DeepEquals, []string{"/dev/mmcblk0p2", "/dev/mmcblk0p3", "/dev/mmcblk0p4"})
 
 		// after ensuring that the nodes exist, we now setup a different, full
 		// device mapping so that later on in the function when we query for
@@ -298,27 +264,33 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 		switch mountCall {
 		case 1:
 			c.Assert(source, Equals, "/dev/mmcblk0p2")
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/2"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p2"))
 			c.Assert(fstype, Equals, "vfat")
 			c.Assert(flags, Equals, uintptr(0))
 			c.Assert(data, Equals, "")
 		case 2:
+			var mntPoint string
 			if opts.encryption {
 				c.Assert(source, Equals, "/dev/mapper/ubuntu-save")
+				mntPoint = "gadget-install/dev-mapper-ubuntu-save"
 			} else {
 				c.Assert(source, Equals, "/dev/mmcblk0p3")
+				mntPoint = "gadget-install/dev-mmcblk0p3"
 			}
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/3"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			c.Assert(fstype, Equals, "ext4")
 			c.Assert(flags, Equals, uintptr(0))
 			c.Assert(data, Equals, "")
 		case 3:
+			var mntPoint string
 			if opts.encryption {
 				c.Assert(source, Equals, "/dev/mapper/ubuntu-data")
+				mntPoint = "gadget-install/dev-mapper-ubuntu-data"
 			} else {
 				c.Assert(source, Equals, "/dev/mmcblk0p4")
+				mntPoint = "gadget-install/dev-mmcblk0p4"
 			}
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/4"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			c.Assert(fstype, Equals, "ext4")
 			c.Assert(flags, Equals, uintptr(0))
 			c.Assert(data, Equals, "")
@@ -335,13 +307,21 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 		umountCall++
 		switch umountCall {
 		case 1:
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/2"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p2"))
 			c.Assert(flags, Equals, 0)
 		case 2:
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/3"))
+			mntPoint := "gadget-install/dev-mmcblk0p3"
+			if opts.encryption {
+				mntPoint = "gadget-install/dev-mapper-ubuntu-save"
+			}
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			c.Assert(flags, Equals, 0)
 		case 3:
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/4"))
+			mntPoint := "gadget-install/dev-mmcblk0p4"
+			if opts.encryption {
+				mntPoint = "gadget-install/dev-mapper-ubuntu-data"
+			}
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			c.Assert(flags, Equals, 0)
 		default:
 			c.Errorf("unexpected umount call (%d)", umountCall)
@@ -357,11 +337,12 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 	var saveEncryptionKey, dataEncryptionKey keys.EncryptionKey
 
 	secbootFormatEncryptedDeviceCall := 0
-	restore = install.MockSecbootFormatEncryptedDevice(func(key keys.EncryptionKey, label, node string) error {
+	restore = install.MockSecbootFormatEncryptedDevice(func(key keys.EncryptionKey, encType secboot.EncryptionType, label, node string) error {
 		if !opts.encryption {
 			c.Error("unexpected call to secboot.FormatEncryptedDevice when encryption is off")
 			return fmt.Errorf("no encryption functions should be called")
 		}
+		c.Check(encType, Equals, secboot.EncryptionTypeLUKS)
 		secbootFormatEncryptedDeviceCall++
 		switch secbootFormatEncryptedDeviceCall {
 		case 1:
@@ -429,11 +410,16 @@ func (s *installSuite) testInstall(c *C, opts installOpts) {
 	}
 	c.Assert(mockPartx.Calls(), DeepEquals, expPartxCalls)
 
-	udevmadmCalls := [][]string{
-		{"udevadm", "settle", "--timeout=180"},
-		{"udevadm", "trigger", "--settle", "/dev/mmcblk0p2"},
+	udevmadmCalls := [][]string{}
+
+	if opts.fromSeed {
+		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "info", "--query", "property", "--name", "/dev/mmcblk0p1"})
+		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "info", "--query", "property", "--name", "/dev/block/42:0"})
 	}
 
+	udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "settle", "--timeout=180"})
+	udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "trigger", "--settle", "/dev/mmcblk0p2"})
+
 	if opts.encryption {
 		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "trigger", "--settle", "/dev/mapper/ubuntu-save"})
 		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "trigger", "--settle", "/dev/mapper/ubuntu-data"})
@@ -567,7 +553,7 @@ func (s *installSuite) TestDeviceFromRoleHappy(c *C) {
 	lv, err := gadgettest.LayoutFromYaml(c.MkDir(), mockUC20GadgetYaml, uc20Mod)
 	c.Assert(err, IsNil)
 
-	device, err := install.DiskWithSystemSeed(lv)
+	device, err := install.DiskWithSystemSeed(lv.Volume)
 	c.Assert(err, IsNil)
 	c.Check(device, Equals, "/dev/fakedevice0")
 }
@@ -577,7 +563,7 @@ func (s *installSuite) TestDeviceFromRoleErrorNoMatchingSysfs(c *C) {
 	lv, err := gadgettest.LayoutFromYaml(c.MkDir(), mockUC20GadgetYaml, uc20Mod)
 	c.Assert(err, IsNil)
 
-	_, err = install.DiskWithSystemSeed(lv)
+	_, err = install.DiskWithSystemSeed(lv.Volume)
 	c.Assert(err, ErrorMatches, `cannot find device for role system-seed: device not found`)
 }
 
@@ -586,7 +572,7 @@ func (s *installSuite) TestDeviceFromRoleErrorNoRole(c *C) {
 	lv, err := gadgettest.LayoutFromYaml(c.MkDir(), mockGadgetYaml, nil)
 	c.Assert(err, IsNil)
 
-	_, err = install.DiskWithSystemSeed(lv)
+	_, err = install.DiskWithSystemSeed(lv.Volume)
 	c.Assert(err, ErrorMatches, "cannot find role system-seed in gadget")
 }
 
@@ -598,6 +584,7 @@ type factoryResetOpts struct {
 	gadgetYaml string
 	traitsJSON string
 	traits     gadget.DiskVolumeDeviceTraits
+	fromSeed   bool
 }
 
 func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
@@ -629,9 +616,28 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 	mockPartx := testutil.MockCommand(c, "partx", "")
 	defer mockPartx.Restore()
 
-	mockUdevadm := testutil.MockCommand(c, "udevadm", "")
+	mockUdevadm := testutil.MockCommand(c, "udevadm", `
+if [ "$*" = "info --query property --name /dev/mmcblk0p1" ]; then
+	echo "ID_PART_ENTRY_DISK=42:0"
+elif [ "$*" = "info --query property --name /dev/block/42:0" ]; then
+	echo "DEVNAME=/dev/mmcblk0"
+	echo "DEVPATH=/devices/virtual/mmcblk0"
+	echo "DEVTYPE=disk"
+	echo "ID_PART_TABLE_UUID=some-gpt-uuid"
+	echo "ID_PART_TABLE_TYPE=GPT"
+fi
+`)
 	defer mockUdevadm.Restore()
 
+	if opts.fromSeed {
+		restoreMountInfo := osutil.MockMountInfo(`130 30 42:1 / /run/mnt/ubuntu-seed rw,relatime shared:54 - vfat /dev/mmcblk0p1 rw
+`)
+		defer restoreMountInfo()
+	} else {
+		restoreMountInfo := osutil.MockMountInfo(``)
+		defer restoreMountInfo()
+	}
+
 	mockCryptsetup := testutil.MockCommand(c, "cryptsetup", "")
 	defer mockCryptsetup.Restore()
 
@@ -647,101 +653,6 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 	if opts.encryption {
 		dataDev = "/dev/mapper/ubuntu-data"
 	}
-	restore = install.MockEnsureNodesExist(func(dss []gadget.OnDiskStructure, timeout time.Duration) error {
-		c.Assert(timeout, Equals, 5*time.Second)
-		expectedDss := []gadget.OnDiskStructure{
-			{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-boot",
-						Label:      "ubuntu-boot",
-						Size:       750 * quantity.SizeMiB,
-						Type:       "0C",
-						Role:       gadget.SystemBoot,
-						Filesystem: "vfat",
-					},
-					StartOffset: (1 + 1200) * quantity.OffsetMiB,
-					YamlIndex:   1,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 2,
-				Node:      "/dev/mmcblk0p2",
-				Size:      750 * quantity.SizeMiB,
-			},
-		}
-		if opts.noSave {
-			// just data
-			expectedDss = append(expectedDss, gadget.OnDiskStructure{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-data",
-						Label:      "ubuntu-data",
-						// TODO: this is set from the yaml, not from the actual
-						// calculated disk size, probably should be updated
-						// somewhere
-						Size:       1500 * quantity.SizeMiB,
-						Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-						Role:       gadget.SystemData,
-						Filesystem: "ext4",
-					},
-					StartOffset: (1 + 1200 + 750) * quantity.OffsetMiB,
-					YamlIndex:   2,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 3,
-				Node:      dataDev,
-				Size:      (30528 - (1 + 1200 + 750)) * quantity.SizeMiB,
-			})
-		} else {
-			// data + save
-			expectedDss = append(expectedDss, []gadget.OnDiskStructure{{
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-save",
-						Label:      "ubuntu-save",
-						Size:       16 * quantity.SizeMiB,
-						Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-						Role:       gadget.SystemSave,
-						Filesystem: "ext4",
-					},
-					StartOffset: (1 + 1200 + 750) * quantity.OffsetMiB,
-					YamlIndex:   2,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 3,
-				Node:      "/dev/mmcblk0p3",
-				Size:      16 * quantity.SizeMiB,
-			}, {
-				LaidOutStructure: gadget.LaidOutStructure{
-					VolumeStructure: &gadget.VolumeStructure{
-						VolumeName: "pi",
-						Name:       "ubuntu-data",
-						Label:      "ubuntu-data",
-						// TODO: this is set from the yaml, not from the actual
-						// calculated disk size, probably should be updated
-						// somewhere
-						Size:       1500 * quantity.SizeMiB,
-						Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-						Role:       gadget.SystemData,
-						Filesystem: "ext4",
-					},
-					StartOffset: (1 + 1200 + 750 + 16) * quantity.OffsetMiB,
-					YamlIndex:   3,
-				},
-				// note this is YamlIndex + 1, the YamlIndex starts at 0
-				DiskIndex: 4,
-				Node:      dataDev,
-				Size:      (30528 - (1 + 1200 + 750 + 16)) * quantity.SizeMiB,
-			}}...)
-		}
-		c.Assert(dss, DeepEquals, expectedDss)
-
-		return nil
-	})
-	defer restore()
 
 	mkfsCall := 0
 	restore = install.MockMkfsMake(func(typ, img, label string, devSize, sectorSize quantity.Size) error {
@@ -781,16 +692,20 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 		switch mountCall {
 		case 1:
 			c.Assert(source, Equals, "/dev/mmcblk0p2")
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/2"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p2"))
 			c.Assert(fstype, Equals, "vfat")
 			c.Assert(flags, Equals, uintptr(0))
 			c.Assert(data, Equals, "")
 		case 2:
 			c.Assert(source, Equals, dataDev)
 			if opts.noSave {
-				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/3"))
+				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p3"))
 			} else {
-				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/4"))
+				mntPoint := "gadget-install/dev-mmcblk0p4"
+				if opts.encryption {
+					mntPoint = "gadget-install/dev-mapper-ubuntu-data"
+				}
+				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			}
 			c.Assert(fstype, Equals, "ext4")
 			c.Assert(flags, Equals, uintptr(0))
@@ -808,13 +723,17 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 		umountCall++
 		switch umountCall {
 		case 1:
-			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/2"))
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p2"))
 			c.Assert(flags, Equals, 0)
 		case 2:
 			if opts.noSave {
-				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/3"))
+				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mmcblk0p3"))
 			} else {
-				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, "gadget-install/4"))
+				mntPoint := "gadget-install/dev-mmcblk0p4"
+				if opts.encryption {
+					mntPoint = "gadget-install/dev-mapper-ubuntu-data"
+				}
+				c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir, mntPoint))
 			}
 			c.Assert(flags, Equals, 0)
 		default:
@@ -830,11 +749,12 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 
 	var dataPrimaryKey keys.EncryptionKey
 	secbootFormatEncryptedDeviceCall := 0
-	restore = install.MockSecbootFormatEncryptedDevice(func(key keys.EncryptionKey, label, node string) error {
+	restore = install.MockSecbootFormatEncryptedDevice(func(key keys.EncryptionKey, encType secboot.EncryptionType, label, node string) error {
 		if !opts.encryption {
 			c.Error("unexpected call to secboot.FormatEncryptedDevice")
 			return fmt.Errorf("unexpected call")
 		}
+		c.Check(encType, Equals, secboot.EncryptionTypeLUKS)
 		secbootFormatEncryptedDeviceCall++
 		switch secbootFormatEncryptedDeviceCall {
 		case 1:
@@ -890,11 +810,16 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 	c.Assert(mockSfdisk.Calls(), HasLen, 0)
 	c.Assert(mockPartx.Calls(), HasLen, 0)
 
-	udevmadmCalls := [][]string{
-		{"udevadm", "trigger", "--settle", "/dev/mmcblk0p2"},
-		{"udevadm", "trigger", "--settle", dataDev},
+	udevmadmCalls := [][]string{}
+
+	if opts.fromSeed {
+		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "info", "--query", "property", "--name", "/dev/mmcblk0p1"})
+		udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "info", "--query", "property", "--name", "/dev/block/42:0"})
 	}
 
+	udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "trigger", "--settle", "/dev/mmcblk0p2"})
+	udevmadmCalls = append(udevmadmCalls, []string{"udevadm", "trigger", "--settle", dataDev})
+
 	c.Assert(mockUdevadm.Calls(), DeepEquals, udevmadmCalls)
 	c.Assert(mkfsCall, Equals, 2)
 	c.Assert(mountCall, Equals, 2)
@@ -927,6 +852,16 @@ func (s *installSuite) testFactoryReset(c *C, opts factoryResetOpts) {
 	c.Assert(mapping2, DeepEquals, mappingOnData)
 }
 
+func (s *installSuite) TestFactoryResetHappyFromSeed(c *C) {
+	s.testFactoryReset(c, factoryResetOpts{
+		disk:       gadgettest.ExpectedRaspiMockDiskMapping,
+		gadgetYaml: gadgettest.RaspiSimplifiedYaml,
+		traitsJSON: gadgettest.ExpectedRaspiDiskVolumeDeviceTraitsJSON,
+		traits:     gadgettest.ExpectedRaspiDiskVolumeDeviceTraits,
+		fromSeed:   true,
+	})
+}
+
 func (s *installSuite) TestFactoryResetHappyWithExisting(c *C) {
 	s.testFactoryReset(c, factoryResetOpts{
 		disk:       gadgettest.ExpectedRaspiMockDiskMapping,
@@ -964,89 +899,19 @@ func (s *installSuite) TestFactoryResetHappyEncrypted(c *C) {
 	})
 }
 
-func mockGadgetPartitionedDisk(c *C) (*gadget.Info, map[string]*gadget.LaidOutVolume, *asserts.Model, string, func()) {
-	cleanups := []func(){}
-	addCleanup := func(r func()) { cleanups = append(cleanups, r) }
-	cleanup := func() {
-		for _, r := range cleanups {
-			r()
-		}
-	}
-
-	// TODO test for UC systems too
-	model := boottest.MakeMockClassicWithModesModel()
-
-	// Create gadget with all files
-	gadgetRoot := filepath.Join(c.MkDir(), "gadget")
-	err := makeMockGadget(gadgetRoot, gadgettest.SingleVolumeClassicWithModesGadgetYaml)
-	c.Assert(err, IsNil)
-	_, allLaidOutVols, err := gadget.LaidOutVolumesFromGadget(gadgetRoot, "", model)
-	c.Assert(err, IsNil)
-
-	ginfo, err := gadget.ReadInfo(gadgetRoot, model)
-	c.Assert(err, IsNil)
-
-	vdaSysPath := "/sys/devices/pci0000:00/0000:00:03.0/virtio1/block/vda"
-	restore := install.MockSysfsPathForBlockDevice(func(device string) (string, error) {
-		c.Assert(strings.HasPrefix(device, "/dev/vda"), Equals, true)
-		return filepath.Join(vdaSysPath, filepath.Base(device)), nil
-	})
-	addCleanup(restore)
-
-	// "Real" disk data that will be read
-	disk := &disks.MockDiskMapping{
-		Structure: []disks.Partition{
-			{
-				PartitionLabel:   "BIOS\x20Boot",
-				KernelDeviceNode: "/dev/vda1",
-				DiskIndex:        1,
-			},
-			{
-				PartitionLabel:   "EFI System partition",
-				KernelDeviceNode: "/dev/vda2",
-				DiskIndex:        2,
-			},
-			{
-				PartitionLabel:   "ubuntu-boot",
-				KernelDeviceNode: "/dev/vda3",
-				DiskIndex:        3,
-			},
-			{
-				PartitionLabel:   "ubuntu-save",
-				KernelDeviceNode: "/dev/vda4",
-				DiskIndex:        4,
-			},
-			{
-				PartitionLabel:   "ubuntu-data",
-				KernelDeviceNode: "/dev/vda5",
-				DiskIndex:        5,
-			},
-		},
-		DiskHasPartitions: true,
-		DevNum:            "disk1",
-		DevNode:           "/dev/vda",
-		DevPath:           vdaSysPath,
-	}
-	diskMapping := map[string]*disks.MockDiskMapping{
-		vdaSysPath: disk,
-		// this simulates a symlink in /sys/block which points to the above path
-		"/sys/block/vda": disk,
-	}
-	restore = disks.MockDevicePathToDiskMapping(diskMapping)
-	addCleanup(restore)
-
-	return ginfo, allLaidOutVols, model, gadgetRoot, cleanup
-}
-
 type writeContentOpts struct {
 	encryption bool
 }
 
 func (s *installSuite) testWriteContent(c *C, opts writeContentOpts) {
-	espMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/2")
-	bootMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/3")
-	saveMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/4")
-	dataMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/5")
+	espMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/dev-vda2")
+	bootMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/dev-vda3")
+	saveMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/dev-vda4")
+	dataMntPt := filepath.Join(dirs.SnapRunDir, "gadget-install/dev-vda5")
+	if opts.encryption {
+		saveMntPt = filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mapper-ubuntu-save")
+		dataMntPt = filepath.Join(dirs.SnapRunDir, "gadget-install/dev-mapper-ubuntu-data")
+	}
 	mountCall := 0
 	restore := install.MockSysMount(func(source, target, fstype string, flags uintptr, data string) error {
 		mountCall++
@@ -1094,18 +959,46 @@ func (s *installSuite) testWriteContent(c *C, opts writeContentOpts) {
 	umountCall := 0
 	restore = install.MockSysUnmount(func(target string, flags int) error {
 		umountCall++
-		if umountCall > 4 {
+		switch umountCall {
+		case 1:
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir,
+				"gadget-install/dev-vda2"))
+		case 2:
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir,
+				"gadget-install/dev-vda3"))
+		case 3:
+			mntPoint := "gadget-install/dev-vda4"
+			if opts.encryption {
+				mntPoint = "gadget-install/dev-mapper-ubuntu-save"
+			}
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir,
+				mntPoint))
+		case 4:
+			mntPoint := "gadget-install/dev-vda5"
+			if opts.encryption {
+				mntPoint = "gadget-install/dev-mapper-ubuntu-data"
+			}
+			c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir,
+				mntPoint))
+		default:
 			c.Errorf("unexpected umount call (%d)", umountCall)
 			return fmt.Errorf("test broken")
 		}
-		c.Assert(target, Equals, filepath.Join(dirs.SnapRunDir,
-			"gadget-install/"+strconv.Itoa(umountCall+1)))
 		c.Assert(flags, Equals, 0)
 		return nil
 	})
 	defer restore()
 
-	ginfo, allLaidOutVols, _, _, restore := mockGadgetPartitionedDisk(c)
+	vdaSysPath := "/sys/devices/pci0000:00/0000:00:03.0/virtio1/block/vda"
+	restore = install.MockSysfsPathForBlockDevice(func(device string) (string, error) {
+		c.Assert(strings.HasPrefix(device, "/dev/vda"), Equals, true)
+		return filepath.Join(vdaSysPath, filepath.Base(device)), nil
+	})
+	defer restore()
+
+	gadgetRoot := filepath.Join(c.MkDir(), "gadget")
+	ginfo, allLaidOutVols, _, restore, err := gadgettest.MockGadgetPartitionedDisk(gadgettest.SingleVolumeClassicWithModesGadgetYaml, gadgetRoot)
+	c.Assert(err, IsNil)
 	defer restore()
 
 	// 10 million mocks later ...
@@ -1123,11 +1016,17 @@ func (s *installSuite) testWriteContent(c *C, opts writeContentOpts) {
 	// Fill encrypted partitions if encrypting
 	var esd *install.EncryptionSetupData
 	if opts.encryption {
-		labelToEncDevice := map[string]string{
-			"ubuntu-save": "/dev/mapper/ubuntu-save",
-			"ubuntu-data": "/dev/mapper/ubuntu-data",
+		labelToEncData := map[string]*install.MockEncryptedDeviceAndRole{
+			"ubuntu-save": {
+				Role:            "system-save",
+				EncryptedDevice: "/dev/mapper/ubuntu-save",
+			},
+			"ubuntu-data": {
+				Role:            "system-data",
+				EncryptedDevice: "/dev/mapper/ubuntu-data",
+			},
 		}
-		esd = install.BuildEncryptionSetupData(labelToEncDevice)
+		esd = install.MockEncryptionSetupData(labelToEncData)
 	}
 	onDiskVols, err := install.WriteContent(ginfo.Volumes, allLaidOutVols, esd, nil, timings.New(nil))
 	c.Assert(err, IsNil)
@@ -1177,8 +1076,35 @@ type encryptPartitionsOpts struct {
 	encryptType secboot.EncryptionType
 }
 
+func expectedCipher() string {
+	switch runtime.GOARCH {
+	case "arm":
+		return "aes-cbc-essiv:sha256"
+	default:
+		return "aes-xts-plain64"
+	}
+}
+
+func expectedKeysize() string {
+	switch runtime.GOARCH {
+	case "arm":
+		return "256"
+	default:
+		return "512"
+	}
+}
+
 func (s *installSuite) testEncryptPartitions(c *C, opts encryptPartitionsOpts) {
-	ginfo, _, model, gadgetRoot, restore := mockGadgetPartitionedDisk(c)
+	vdaSysPath := "/sys/devices/pci0000:00/0000:00:03.0/virtio1/block/vda"
+	restore := install.MockSysfsPathForBlockDevice(func(device string) (string, error) {
+		c.Assert(strings.HasPrefix(device, "/dev/vda"), Equals, true)
+		return filepath.Join(vdaSysPath, filepath.Base(device)), nil
+	})
+	defer restore()
+
+	gadgetRoot := filepath.Join(c.MkDir(), "gadget")
+	ginfo, _, model, restore, err := gadgettest.MockGadgetPartitionedDisk(gadgettest.SingleVolumeClassicWithModesGadgetYaml, gadgetRoot)
+	c.Assert(err, IsNil)
 	defer restore()
 
 	mockCryptsetup := testutil.MockCommand(c, "cryptsetup", "")
@@ -1206,10 +1132,10 @@ func (s *installSuite) testEncryptPartitions(c *C, opts encryptPartitionsOpts) {
 	c.Assert(err, IsNil)
 
 	c.Assert(mockCryptsetup.Calls(), DeepEquals, [][]string{
-		{"cryptsetup", "-q", "luksFormat", "--type", "luks2", "--key-file", "-", "--cipher", "aes-xts-plain64", "--key-size", "512", "--label", "ubuntu-save-enc", "--pbkdf", "argon2i", "--pbkdf-force-iterations", "4", "--pbkdf-memory", "32", "--luks2-metadata-size", "2048k", "--luks2-keyslots-size", "2560k", "/dev/vda4"},
+		{"cryptsetup", "-q", "luksFormat", "--type", "luks2", "--key-file", "-", "--cipher", expectedCipher(), "--key-size", expectedKeysize(), "--label", "ubuntu-save-enc", "--pbkdf", "argon2i", "--pbkdf-force-iterations", "4", "--pbkdf-memory", "32", "--luks2-metadata-size", "2048k", "--luks2-keyslots-size", "2560k", "/dev/vda4"},
 		{"cryptsetup", "config", "--priority", "prefer", "--key-slot", "0", "/dev/vda4"},
 		{"cryptsetup", "open", "--key-file", "-", "/dev/vda4", "ubuntu-save"},
-		{"cryptsetup", "-q", "luksFormat", "--type", "luks2", "--key-file", "-", "--cipher", "aes-xts-plain64", "--key-size", "512", "--label", "ubuntu-data-enc", "--pbkdf", "argon2i", "--pbkdf-force-iterations", "4", "--pbkdf-memory", "32", "--luks2-metadata-size", "2048k", "--luks2-keyslots-size", "2560k", "/dev/vda5"},
+		{"cryptsetup", "-q", "luksFormat", "--type", "luks2", "--key-file", "-", "--cipher", expectedCipher(), "--key-size", expectedKeysize(), "--label", "ubuntu-data-enc", "--pbkdf", "argon2i", "--pbkdf-force-iterations", "4", "--pbkdf-memory", "32", "--luks2-metadata-size", "2048k", "--luks2-keyslots-size", "2560k", "/dev/vda5"},
 		{"cryptsetup", "config", "--priority", "prefer", "--key-slot", "0", "/dev/vda5"},
 		{"cryptsetup", "open", "--key-file", "-", "/dev/vda5", "ubuntu-data"},
 	})
@@ -1222,7 +1148,16 @@ func (s *installSuite) TestInstallEncryptPartitionsLUKSHappy(c *C) {
 }
 
 func (s *installSuite) TestInstallEncryptPartitionsNoDeviceSet(c *C) {
-	ginfo, _, model, gadgetRoot, restore := mockGadgetPartitionedDisk(c)
+	vdaSysPath := "/sys/devices/pci0000:00/0000:00:03.0/virtio1/block/vda"
+	restore := install.MockSysfsPathForBlockDevice(func(device string) (string, error) {
+		c.Assert(strings.HasPrefix(device, "/dev/vda"), Equals, true)
+		return filepath.Join(vdaSysPath, filepath.Base(device)), nil
+	})
+	defer restore()
+
+	gadgetRoot := filepath.Join(c.MkDir(), "gadget")
+	ginfo, _, model, restore, err := gadgettest.MockGadgetPartitionedDisk(gadgettest.SingleVolumeClassicWithModesGadgetYaml, gadgetRoot)
+	c.Assert(err, IsNil)
 	defer restore()
 
 	encryptSetup, err := install.EncryptPartitions(ginfo.Volumes, secboot.EncryptionTypeLUKS, model, gadgetRoot, "", timings.New(nil))
