@@ -23,6 +23,7 @@ import (
 	"fmt"
 
 	"github.com/snapcore/snapd/gadget/quantity"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil/disks"
 )
 
@@ -30,7 +31,25 @@ import (
 
 // OnDiskStructure represents a gadget structure laid on a block device.
 type OnDiskStructure struct {
-	LaidOutStructure
+	// Name, when non empty, provides the name of the structure
+	Name string
+	// PartitionFSLabel provides the filesystem label
+	PartitionFSLabel string
+	// Type of the structure, which can be 2-hex digit MBR partition,
+	// 36-char GUID partition, comma separated <mbr>,<guid> for hybrid
+	// partitioning schemes, or 'bare' when the structure is not considered
+	// a partition.
+	//
+	// For backwards compatibility type 'mbr' is also accepted, and the
+	// structure is treated as if it is of role 'mbr'.
+	Type string
+	// PartitionFSType used for the partition filesystem: 'vfat', 'ext4',
+	// 'none' for structures of type 'bare', or 'crypto_LUKS' for encrypted
+	// partitions.
+	PartitionFSType string
+	// StartOffset defines the start offset of the structure within the
+	// enclosing volume
+	StartOffset quantity.Offset
 
 	// Node identifies the device node of the block device.
 	Node string
@@ -154,22 +173,18 @@ func OnDiskStructureFromPartition(p disks.Partition) (OnDiskStructure, error) {
 		return OnDiskStructure{}, fmt.Errorf("cannot decode filesystem label for partition %s: %v", p.KernelDeviceNode, err)
 	}
 
-	volStruct := VolumeStructure{
-		Name:       decodedPartLabel,
-		Size:       quantity.Size(p.SizeInBytes),
-		Label:      decodedFsLabel,
-		Type:       p.PartitionType,
-		Filesystem: p.FilesystemType,
-		ID:         p.PartitionUUID,
-	}
+	logger.Debugf("OnDiskStructureFromPartition: p.FilesystemType %q, p.FilesystemLabel %q",
+		p.FilesystemType, p.FilesystemLabel)
 
+	// TODO add ID in second part of the gadget refactoring?
 	return OnDiskStructure{
-		LaidOutStructure: LaidOutStructure{
-			VolumeStructure: &volStruct,
-			StartOffset:     quantity.Offset(p.StartInBytes),
-		},
-		DiskIndex: int(p.DiskIndex),
-		Size:      quantity.Size(p.SizeInBytes),
-		Node:      p.KernelDeviceNode,
+		Name:             decodedPartLabel,
+		PartitionFSLabel: decodedFsLabel,
+		Type:             p.PartitionType,
+		PartitionFSType:  p.FilesystemType,
+		StartOffset:      quantity.Offset(p.StartInBytes),
+		DiskIndex:        int(p.DiskIndex),
+		Size:             quantity.Size(p.SizeInBytes),
+		Node:             p.KernelDeviceNode,
 	}, nil
 }
