@@ -811,7 +811,7 @@ var (
 		"ppp":                       {"core"},
 		"pulseaudio":                {"app", "core"},
 		"pwm":                       {"core", "gadget"},
-		"qualcomm-ipc-router":       {"core"},
+		"qualcomm-ipc-router":       {"core", "app"},
 		"raw-volume":                {"core", "gadget"},
 		"scsi-generic":              {"core"},
 		"sd-control":                {"core"},
@@ -828,6 +828,7 @@ var (
 		"unity8-calendar":           {"app"},
 		"unity8-contacts":           {"app"},
 		"upower-observe":            {"app", "core"},
+		"userns":                    {"core"},
 		"wayland":                   {"app", "core"},
 		"x11":                       {"app", "core"},
 		// snowflakes
@@ -836,6 +837,7 @@ var (
 		"docker":          nil,
 		"lxd":             nil,
 		"microceph":       nil,
+		"microovn":        nil,
 		"pkcs11":          nil,
 		"posix-mq":        nil,
 		"shared-memory":   nil,
@@ -897,11 +899,17 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 	c.Assert(err, Not(IsNil))
 	c.Assert(err, ErrorMatches, "installation not allowed by \"microceph\" slot rule of interface \"microceph\"")
 
+	// test microovn specially
+	ic = s.installSlotCand(c, "microovn", snap.TypeApp, ``)
+	err = ic.Check()
+	c.Assert(err, Not(IsNil))
+	c.Assert(err, ErrorMatches, "installation not allowed by \"microovn\" slot rule of interface \"microovn\"")
+
 	// test shared-memory specially
 	ic = s.installSlotCand(c, "shared-memory", snap.TypeApp, ``)
 	err = ic.Check()
 	c.Assert(err, Not(IsNil))
-	c.Assert(err, ErrorMatches, "installation not allowed by \"shared-memory\" slot rule of interface \"shared-memory\"")
+	c.Assert(err, ErrorMatches, "installation denied by \"shared-memory\" slot rule of interface \"shared-memory\"")
 
 	// The core and snapd snaps may provide a shared-memory slot
 	ic = s.installSlotCand(c, "shared-memory", snap.TypeOS, `name: core
@@ -921,40 +929,62 @@ slots:
 `)
 	ic.SnapDeclaration = s.mockSnapDecl(c, "snapd", "PMrrV4ml8uWuEUDBT8dSGnKUYbevVhc4", "canonical", "")
 	c.Assert(ic.Check(), IsNil)
+
+	ic = s.installSlotCand(c, "udisks2", snap.TypeApp, `name: udisks2
+version: 0
+type: app
+slots:
+  udisks2:
+`)
+	err = ic.Check()
+	c.Assert(err, IsNil)
+
+	ic = s.installSlotCand(c, "udisks2", snap.TypeApp, `name: udisks2
+version: 0
+type: app
+slots:
+  udisks2:
+    udev-file: some/file
+`)
+	err = ic.Check()
+	c.Assert(err, Not(IsNil))
+	c.Assert(err, ErrorMatches, "installation not allowed by \"udisks2\" slot rule of interface \"udisks2\"")
 }
 
 func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 	all := builtin.Interfaces()
 
 	restricted := map[string]bool{
-		"block-devices":         true,
-		"classic-support":       true,
-		"desktop-launch":        true,
-		"dm-crypt":              true,
-		"docker-support":        true,
-		"greengrass-support":    true,
-		"gpio-control":          true,
-		"ion-memory-control":    true,
-		"kernel-module-control": true,
-		"kernel-module-load":    true,
-		"kubernetes-support":    true,
-		"lxd-support":           true,
-		"microstack-support":    true,
-		"mount-control":         true,
-		"multipass-support":     true,
-		"packagekit-control":    true,
-		"personal-files":        true,
-		"polkit":                true,
-		"sd-control":            true,
-		"snap-refresh-control":  true,
-		"snap-themes-control":   true,
-		"snapd-control":         true,
-		"steam-support":         true,
-		"system-files":          true,
-		"tee":                   true,
-		"uinput":                true,
-		"unity8":                true,
-		"xilinx-dma":            true,
+		"block-devices":          true,
+		"classic-support":        true,
+		"desktop-launch":         true,
+		"dm-crypt":               true,
+		"docker-support":         true,
+		"greengrass-support":     true,
+		"gpio-control":           true,
+		"ion-memory-control":     true,
+		"kernel-module-control":  true,
+		"kernel-module-load":     true,
+		"kubernetes-support":     true,
+		"lxd-support":            true,
+		"microstack-support":     true,
+		"mount-control":          true,
+		"multipass-support":      true,
+		"nvidia-drivers-support": true,
+		"packagekit-control":     true,
+		"personal-files":         true,
+		"polkit":                 true,
+		"sd-control":             true,
+		"snap-refresh-control":   true,
+		"snap-themes-control":    true,
+		"snapd-control":          true,
+		"steam-support":          true,
+		"system-files":           true,
+		"tee":                    true,
+		"uinput":                 true,
+		"unity8":                 true,
+		"userns":                 true,
+		"xilinx-dma":             true,
 	}
 
 	for _, iface := range all {
@@ -1003,6 +1033,7 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 		"lxd":                       true,
 		"maliit":                    true,
 		"microceph":                 true,
+		"microovn":                  true,
 		"mir":                       true,
 		"online-accounts-service":   true,
 		"posix-mq":                  true,
@@ -1101,7 +1132,6 @@ func (s *baseDeclSuite) TestConnectionImplicitOnClassicOrAppSnap(c *C) {
 		"network-manager": true,
 		"ofono":           true,
 		"pulseaudio":      true,
-		"upower-observe":  true,
 	}
 
 	for _, iface := range all {
@@ -1174,42 +1204,45 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 	// given how the rules work this can be delicate,
 	// listed here to make sure that was a conscious decision
 	bothSides := map[string]bool{
-		"block-devices":         true,
-		"audio-playback":        true,
-		"classic-support":       true,
-		"core-support":          true,
-		"custom-device":         true,
-		"desktop-launch":        true,
-		"dm-crypt":              true,
-		"docker-support":        true,
-		"greengrass-support":    true,
-		"gpio-control":          true,
-		"ion-memory-control":    true,
-		"kernel-module-control": true,
-		"kernel-module-load":    true,
-		"kubernetes-support":    true,
-		"lxd-support":           true,
-		"microstack-support":    true,
-		"mount-control":         true,
-		"multipass-support":     true,
-		"packagekit-control":    true,
-		"personal-files":        true,
-		"pkcs11":                true,
-		"posix-mq":              true,
-		"polkit":                true,
-		"sd-control":            true,
-		"shared-memory":         true,
-		"snap-refresh-control":  true,
-		"snap-themes-control":   true,
-		"snapd-control":         true,
-		"steam-support":         true,
-		"system-files":          true,
-		"tee":                   true,
-		"udisks2":               true,
-		"uinput":                true,
-		"unity8":                true,
-		"wayland":               true,
-		"xilinx-dma":            true,
+		"block-devices":          true,
+		"audio-playback":         true,
+		"classic-support":        true,
+		"core-support":           true,
+		"custom-device":          true,
+		"desktop-launch":         true,
+		"dm-crypt":               true,
+		"docker-support":         true,
+		"greengrass-support":     true,
+		"gpio-control":           true,
+		"ion-memory-control":     true,
+		"kernel-module-control":  true,
+		"kernel-module-load":     true,
+		"kubernetes-support":     true,
+		"lxd-support":            true,
+		"microstack-support":     true,
+		"mount-control":          true,
+		"multipass-support":      true,
+		"nvidia-drivers-support": true,
+		"packagekit-control":     true,
+		"personal-files":         true,
+		"pkcs11":                 true,
+		"posix-mq":               true,
+		"polkit":                 true,
+		"qualcomm-ipc-router":    true,
+		"sd-control":             true,
+		"shared-memory":          true,
+		"snap-refresh-control":   true,
+		"snap-themes-control":    true,
+		"snapd-control":          true,
+		"steam-support":          true,
+		"system-files":           true,
+		"tee":                    true,
+		"udisks2":                true,
+		"uinput":                 true,
+		"unity8":                 true,
+		"userns":                 true,
+		"wayland":                true,
+		"xilinx-dma":             true,
 	}
 
 	for _, iface := range all {
@@ -1288,6 +1321,72 @@ plugs:
 	c.Check(err, NotNil)
 }
 
+func (s *baseDeclSuite) TestConnectionQualcommIpcRouter(c *C) {
+	// we let connect explicitly as long as qcipc matches
+
+	slotDecl1 := s.mockSnapDecl(c, "slot-snap", "slot-snap-id", "pub1", "")
+	plugDecl1 := s.mockSnapDecl(c, "plug-snap", "plug-snap-id", "pub1", "")
+
+	// Same qcipc label
+	cand := s.connectCand(c, "qc-router", `name: slot-snap
+version: 0
+slots:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+    address: abcd
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+`)
+	cand.SlotSnapDeclaration = slotDecl1
+	cand.PlugSnapDeclaration = plugDecl1
+	err := cand.Check()
+	c.Check(err, IsNil)
+
+	// Different qcipc label
+	cand = s.connectCand(c, "qc-router", `name: slot-snap
+version: 0
+slots:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: monitor
+    address: abcd
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qc-router:
+    interface: qualcomm-ipc-router
+    qcipc: other
+`)
+	cand.SlotSnapDeclaration = slotDecl1
+	cand.PlugSnapDeclaration = plugDecl1
+	err = cand.Check()
+	c.Check(err.Error(), Equals, `connection not allowed by slot rule of interface "qualcomm-ipc-router"`)
+
+	// Legacy case with slot provided by system
+	cand = s.connectCand(c, "qualcomm-ipc-router", `name: snapd
+version: 0
+type: snapd
+slots:
+  qualcomm-ipc-router:
+`, `
+name: plug-snap
+version: 0
+plugs:
+  qualcomm-ipc-router:
+`)
+	cand.SlotSnapDeclaration = s.mockSnapDecl(c, "snapd", "PMrrV4ml8uWuEUDBT8dSGnKUYbevVhc4", "canonical", "")
+	cand.PlugSnapDeclaration = plugDecl1
+	err = cand.Check()
+	c.Check(err, IsNil)
+}
+
 func (s *baseDeclSuite) TestConnectionSharedMemory(c *C) {
 	// we let connect explicitly as long as shared-memory matches
 
