@@ -34,6 +34,7 @@ import (
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/assertstate"
@@ -122,37 +123,32 @@ func (m *DeviceManager) doGenerateDeviceKey(t *state.Task, _ *tomb.Tomb) error {
 	return nil
 }
 
-func newEnoughProxy(st *state.State, proxyURL *url.URL, client *http.Client) bool {
+func newEnoughProxy(st *state.State, proxyURL *url.URL, client *http.Client) (bool, error) {
 	st.Unlock()
 	defer st.Lock()
 
-	const prefix = "Cannot check whether proxy store supports a custom serial vault"
+	const prefix = "cannot check whether proxy store supports a custom serial vault"
 
 	req, err := http.NewRequest("HEAD", proxyURL.String(), nil)
 	if err != nil {
-		// can't really happen unless proxyURL is somehow broken
-		logger.Debugf(prefix+": %v", err)
-		return false
+		return false, fmt.Errorf(prefix+": %v", err)
 	}
 	req.Header.Set("User-Agent", snapdenv.UserAgent())
 	resp, err := client.Do(req)
 	if err != nil {
 		// some sort of network or protocol error
-		logger.Debugf(prefix+": %v", err)
-		return false
+		return false, fmt.Errorf(prefix+": %v", err)
 	}
 	resp.Body.Close()
 	if resp.StatusCode != 200 {
-		logger.Debugf(prefix+": Head request returned %s.", resp.Status)
-		return false
+		return false, fmt.Errorf(prefix+": Head request returned %s.", resp.Status)
 	}
 	verstr := resp.Header.Get("Snap-Store-Version")
 	ver, err := strconv.Atoi(verstr)
 	if err != nil {
-		logger.Debugf(prefix+": Bogus Snap-Store-Version header %q.", verstr)
-		return false
+		return false, fmt.Errorf(prefix+": Bogus Snap-Store-Version header %q.", verstr)
 	}
-	return ver >= 6
+	return ver >= 6, nil
 }
 
 func (cfg *serialRequestConfig) setURLs(proxyURL, svcURL *url.URL) {
@@ -248,7 +244,7 @@ func (rc *initialRegistrationContext) FinishRegistration(serial *asserts.Serial)
 // registrationCtx returns a registrationContext appropriate for the task and its change.
 func (m *DeviceManager) registrationCtx(t *state.Task) (registrationContext, error) {
 	remodCtx, err := remodelCtxFromTask(t)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	}
 	if regCtx, ok := remodCtx.(registrationContext); ok {
@@ -316,7 +312,7 @@ func prepareSerialRequest(t *state.Task, regCtx registrationContext, privKey ass
 	// slower full retries
 	var nTentatives int
 	err := t.Get("pre-poll-tentatives", &nTentatives)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return "", err
 	}
 	nTentatives++
@@ -335,11 +331,6 @@ func prepareSerialRequest(t *state.Task, regCtx registrationContext, privKey ass
 
 	resp, err := client.Do(req)
 	if err != nil {
-		if !httputil.ShouldRetryError(err) {
-			// a non temporary net error fully errors out and triggers a retry
-			// retries
-			return "", fmt.Errorf("cannot retrieve request-id for making a request for a serial: %v", err)
-		}
 		if httputil.NoNetwork(err) {
 			// If there is no network there is no need to count
 			// this as a tentatives attempt. If we do it this
@@ -358,6 +349,30 @@ func prepareSerialRequest(t *state.Task, regCtx registrationContext, privKey ass
 			noNetworkRetryInterval := retryInterval / 2
 			return "", &state.Retry{After: noNetworkRetryInterval}
 		}
+		if httputil.IsCertExpiredOrNotValidYetError(err) {
+			// If the cert is expired/not-valid yet that
+			// most likely means that the devices has no
+			// ntp-synced time yet. We will retry for up
+			// to 2048s (timesyncd.conf(5) says the
+			// maximum poll time is 2048s which is
+			// 34min8s). With retry of 60s the below adds
+			// up to 37.5m.
+			switch {
+			case nTentatives <= 5:
+				return "", &state.Retry{After: retryInterval / 2}
+			case nTentatives <= 10:
+				return "", &state.Retry{After: retryInterval}
+			case nTentatives <= 15:
+				return "", &state.Retry{After: retryInterval * 2}
+			case nTentatives <= 20:
+				return "", &state.Retry{After: retryInterval * 4}
+			}
+		}
+		if !httputil.ShouldRetryError(err) {
+			// a non temporary net error fully errors out and triggers a retry
+			// retries
+			return "", fmt.Errorf("cannot retrieve request-id for making a request for a serial: %v", err)
+		}
 
 		return "", retryErr(t, nTentatives, "cannot retrieve request-id for making a request for a serial: %v", err)
 	}
@@ -484,7 +499,7 @@ var httputilNewHTTPClient = httputil.NewHTTPClient
 func getSerial(t *state.Task, regCtx registrationContext, privKey asserts.PrivateKey, device *auth.DeviceState, tm timings.Measurer) (serial *asserts.Serial, ancillaryBatch *asserts.Batch, err error) {
 	var serialSup serialSetup
 	err = t.Get("serial-setup", &serialSup)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, nil, err
 	}
 
@@ -504,6 +519,9 @@ func getSerial(t *state.Task, regCtx registrationContext, privKey asserts.Privat
 		MayLogBody:         true,
 		Proxy:              proxyConf.Conf,
 		ProxyConnectHeader: http.Header{"User-Agent": []string{snapdenv.UserAgent()}},
+		ExtraSSLCerts: &httputil.ExtraSSLCertsFromDir{
+			Dir: dirs.SnapdStoreSSLCertsDir,
+		},
 	})
 
 	cfg, err := getSerialRequestConfig(t, regCtx, client)
@@ -585,7 +603,7 @@ func getSerialRequestConfig(t *state.Task, regCtx registrationContext, client *h
 
 	st := t.State()
 	tr := config.NewTransaction(st)
-	if proxyStore, err := proxyStore(st, tr); err != nil && err != state.ErrNoState {
+	if proxyStore, err := proxyStore(st, tr); err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	} else if proxyStore != nil {
 		proxyURL = proxyStore.URL()
@@ -636,9 +654,24 @@ func getSerialRequestConfig(t *state.Task, regCtx registrationContext, client *h
 		}
 	}
 
-	if proxyURL != nil && svcURL != nil && !newEnoughProxy(st, proxyURL, client) {
-		logger.Noticef("Proxy store does not support custom serial vault; ignoring the proxy")
-		proxyURL = nil
+	if proxyURL != nil && svcURL != nil {
+		newEnough, err := newEnoughProxy(st, proxyURL, client)
+		if err != nil {
+			// Ignore the proxy on any error for
+			// compatibility with previous versions of
+			// snapd.
+			//
+			// TODO: provide a way for the users to specify
+			// if they want to use the proxy store for their
+			// device-service.url or not. This needs design.
+			// (see LP:#2023166)
+			logger.Noticef("cannot reach proxy store: %v; ignore the proxy", err)
+			proxyURL = nil
+		}
+		if !newEnough {
+			logger.Noticef("Proxy store does not support custom serial vault; ignoring the proxy")
+			proxyURL = nil
+		}
 	}
 
 	cfg.setURLs(proxyURL, svcURL)
@@ -666,7 +699,7 @@ func (m *DeviceManager) doRequestSerial(t *state.Task, _ *tomb.Tomb) error {
 
 	// NB: the keyPair is fixed for now
 	privKey, err := m.keyPair()
-	if err == state.ErrNoState {
+	if errors.Is(err, state.ErrNoState) {
 		return fmt.Errorf("internal error: cannot find device key pair")
 	}
 	if err != nil {
@@ -680,7 +713,7 @@ func (m *DeviceManager) doRequestSerial(t *state.Task, _ *tomb.Tomb) error {
 		"model":               device.Model,
 		"device-key-sha3-384": privKey.PublicKey().ID(),
 	})
-	if err != nil && !asserts.IsNotFound(err) {
+	if err != nil && !errors.Is(err, &asserts.NotFoundError{}) {
 		return err
 	}
 
