@@ -123,6 +123,8 @@ type autoRefresh struct {
 	nextRefresh         time.Time
 	lastRefreshAttempt  time.Time
 	managedDeniedLogged bool
+
+	restoredMonitoring bool
 }
 
 func newAutoRefresh(st *state.State) *autoRefresh {
@@ -266,10 +268,12 @@ func (m *autoRefresh) canRefreshRespectingMetered(now, lastRefresh time.Time) (c
 }
 
 // Ensure ensures that we refresh all installed snaps periodically
-func (m *autoRefresh) Ensure() error {
+func (m *autoRefresh) Ensure() (err error) {
 	m.state.Lock()
 	defer m.state.Unlock()
 
+	m.restoreMonitoring()
+
 	// see if it even makes sense to try to refresh
 	if CanAutoRefresh == nil {
 		return nil
@@ -278,6 +282,27 @@ func (m *autoRefresh) Ensure() error {
 		return err
 	}
 
+	// is there a previously partially inhibited auto-refresh that can now be continued?
+	if attempt, ok := canContinueAutoRefresh(m.state); ok {
+		// override the auto-refresh delay if we're continuing an inhibited auto-refresh
+		// for the first time (because the snap just closed after we notified the user)
+		overrideDelay := attempt == 1
+		err := m.launchAutoRefresh(overrideDelay)
+		if err != nil {
+			if errors.Is(err, tooSoonError{}) {
+				// ignore error, retry the auto-refresh later
+				return nil
+			}
+
+			// we didn't auto-refresh, so keep flag but increase attempt counter
+			m.state.Cache("auto-refresh-continue-attempt", attempt+1)
+			return err
+		}
+		// clear the continue flag if the auto-refresh was scheduled successfully
+		m.state.Cache("auto-refresh-continue-attempt", nil)
+		return nil
+	}
+
 	// get lastRefresh and schedule
 	lastRefresh, err := m.LastRefresh()
 	if err != nil {
@@ -360,23 +385,64 @@ func (m *autoRefresh) Ensure() error {
 				return nil
 			}
 
-			// Check that we have reasonable delays between attempts.
-			// If the store is under stress we need to make sure we do not
-			// hammer it too often
-			if !m.lastRefreshAttempt.IsZero() && m.lastRefreshAttempt.Add(refreshRetryDelay).After(time.Now()) {
+			overrideDelay := false
+			err = m.launchAutoRefresh(overrideDelay)
+			if _, ok := err.(*httputil.PersistentNetworkError); ok {
+				// refresh will be retried after refreshRetryDelay
+				return err
+			} else if errors.Is(err, tooSoonError{}) {
+				// ignore error, retry the auto-refresh later
 				return nil
 			}
 
-			err = m.launchAutoRefresh()
-			if _, ok := err.(*httputil.PersistentNetworkError); !ok {
-				m.nextRefresh = time.Time{}
-			} // else - refresh will be retried after refreshRetryDelay
+			// refreshed or hit an non-persistent network error, so reset nextRefresh
+			m.nextRefresh = time.Time{}
 		}
 	}
 
 	return err
 }
 
+func canContinueAutoRefresh(st *state.State) (int, bool) {
+	if cachedAttempt := st.Cached("auto-refresh-continue-attempt"); cachedAttempt != nil {
+		return cachedAttempt.(int), true
+	}
+
+	return 0, false
+}
+
+func (m *autoRefresh) restoreMonitoring() {
+	if m.restoredMonitoring {
+		return
+	}
+
+	var monitored []string
+	if err := m.state.Get("monitored-snaps", &monitored); err != nil && !errors.Is(err, state.ErrNoState) {
+		logger.Noticef("cannot restore monitoring: %v", err)
+		return
+	}
+
+	defer func() { m.restoredMonitoring = true }()
+	if len(monitored) == 0 {
+		return
+	}
+
+	monitoring := make(map[string]chan<- bool)
+	for _, snap := range monitored {
+		done := make(chan string, 1)
+		if err := cgroupMonitorSnapEnded(snap, done); err != nil {
+			logger.Noticef("cannot restore monitoring for snap %q closure: %v", snap, err)
+			continue
+		}
+
+		abort := make(chan bool, 1)
+		monitoring[snap] = abort
+
+		go continueRefreshOnSnapClose(m.state, snap, done, abort)
+	}
+	updateMonitoringState(m.state, monitoring)
+}
+
 // isRefreshHeld returns whether an auto-refresh is currently held back or not,
 // as indicated by m.EffectiveRefreshHold().
 func (m *autoRefresh) isRefreshHeld() (bool, time.Time, error) {
@@ -503,8 +569,29 @@ func autoRefreshSummary(updated []string) string {
 	return msg
 }
 
+type tooSoonError struct{}
+
+func (e tooSoonError) Error() string {
+	return "cannot auto-refresh so soon"
+}
+
+func (tooSoonError) Is(err error) bool {
+	_, ok := err.(tooSoonError)
+	return ok
+}
+
 // launchAutoRefresh creates the auto-refresh taskset and a change for it.
-func (m *autoRefresh) launchAutoRefresh() error {
+func (m *autoRefresh) launchAutoRefresh(overrideDelay bool) error {
+	// Check that we have reasonable delays between attempts.
+	// If the store is under stress we need to make sure we do not
+	// hammer it too often
+	now := timeNow()
+	minAttempt := m.lastRefreshAttempt.Add(refreshRetryDelay)
+	if !overrideDelay && !m.lastRefreshAttempt.IsZero() && minAttempt.After(now) {
+		return tooSoonError{}
+	}
+	m.lastRefreshAttempt = now
+
 	perfTimings := timings.New(map[string]string{"ensure": "auto-refresh"})
 	tm := perfTimings.StartSpan("auto-refresh", "query store and setup auto-refresh change")
 	defer func() {
@@ -512,10 +599,8 @@ func (m *autoRefresh) launchAutoRefresh() error {
 		perfTimings.Save(m.state)
 	}()
 
-	m.lastRefreshAttempt = time.Now()
-
 	// NOTE: this will unlock and re-lock state for network ops
-	updated, tasksets, err := AutoRefresh(auth.EnsureContextTODO(), m.state)
+	updated, updateTss, err := AutoRefresh(auth.EnsureContextTODO(), m.state, &AutoRefreshOptions{IsContinuedAutoRefresh: overrideDelay})
 
 	// TODO: we should have some way to lock just creating and starting changes,
 	//       as that would alleviate this race condition we are guarding against
@@ -540,12 +625,20 @@ func (m *autoRefresh) launchAutoRefresh() error {
 		logger.Noticef("Cannot prepare auto-refresh change due to a permanent network error: %s", err)
 		return err
 	}
-	m.state.Set("last-refresh", time.Now())
+	m.state.Set("last-refresh", timeNow())
 	if err != nil {
 		logger.Noticef("Cannot prepare auto-refresh change: %s", err)
 		return err
 	}
 
+	if _, err := createPreDownloadChange(m.state, updateTss); err != nil {
+		return err
+	}
+
+	if len(updateTss.Refresh) == 0 {
+		return nil
+	}
+
 	msg := autoRefreshSummary(updated)
 	if msg == "" {
 		logger.Noticef(i18n.G("auto-refresh: all snaps are up-to-date"))
@@ -553,7 +646,7 @@ func (m *autoRefresh) launchAutoRefresh() error {
 	}
 
 	chg := m.state.NewChange("auto-refresh", msg)
-	for _, ts := range tasksets {
+	for _, ts := range updateTss.Refresh {
 		chg.AddAll(ts)
 	}
 	chg.Set("snap-names", updated)
@@ -563,6 +656,30 @@ func (m *autoRefresh) launchAutoRefresh() error {
 	return nil
 }
 
+// createPreDownloadChange creates a pre-download change if any relevant tasksets
+// exist in the UpdateTaskSets and returns whether or not a change was created.
+func createPreDownloadChange(st *state.State, updateTss *UpdateTaskSets) (bool, error) {
+	if updateTss != nil && len(updateTss.PreDownload) > 0 {
+		var snapNames []string
+		for _, ts := range updateTss.PreDownload {
+			task := ts.Tasks()[0]
+			var snapsup *SnapSetup
+			if err := task.Get("snap-setup", &snapsup); err != nil {
+				return false, err
+			}
+			snapNames = append(snapNames, snapsup.InstanceName())
+		}
+
+		chgSummary := fmt.Sprintf(i18n.G("Pre-download %s for auto-refresh"), strutil.Quoted(snapNames))
+		preDlChg := st.NewChange("pre-download", chgSummary)
+		for _, ts := range updateTss.PreDownload {
+			preDlChg.AddAll(ts)
+		}
+		return true, nil
+	}
+	return false, nil
+}
+
 func autoRefreshInFlight(st *state.State) bool {
 	for _, chg := range st.Changes() {
 		if chg.Kind() == "auto-refresh" && !chg.Status().Ready() {
@@ -587,6 +704,7 @@ func getTime(st *state.State, timeKey string) (time.Time, error) {
 // This allows the, possibly slow, communication with each snapd session agent,
 // to be performed without holding the snap state lock.
 var asyncPendingRefreshNotification = func(context context.Context, client *userclient.Client, refreshInfo *userclient.PendingSnapRefreshInfo) {
+	logger.Debugf("notifying agents about pending refresh for snap %q", refreshInfo.InstanceName)
 	go func() {
 		if err := client.PendingRefreshNotification(context, refreshInfo); err != nil {
 			logger.Noticef("Cannot send notification about pending refresh: %v", err)
@@ -594,25 +712,44 @@ var asyncPendingRefreshNotification = func(context context.Context, client *user
 	}()
 }
 
+type timedBusySnapError struct {
+	err           *BusySnapError
+	timeRemaining time.Duration
+}
+
+func (e *timedBusySnapError) PendingSnapRefreshInfo() *userclient.PendingSnapRefreshInfo {
+	refreshInfo := e.err.PendingSnapRefreshInfo()
+	refreshInfo.TimeRemaining = e.timeRemaining
+	return refreshInfo
+}
+
+func (e *timedBusySnapError) Error() string {
+	return e.err.Error()
+}
+
+func (e *timedBusySnapError) Is(err error) bool {
+	_, ok := err.(*timedBusySnapError)
+	return ok
+}
+
 // inhibitRefresh returns an error if refresh is inhibited by running apps.
 //
 // Internally the snap state is updated to remember when the inhibition first
 // took place. Apps can inhibit refreshes for up to "maxInhibition", beyond
 // that period the refresh will go ahead despite application activity.
-func inhibitRefresh(st *state.State, snapst *SnapState, info *snap.Info, checker func(*snap.Info) error) error {
-	checkerErr := checker(info)
+func inhibitRefresh(st *state.State, snapst *SnapState, snapsup *SnapSetup, info *snap.Info) error {
+	checkerErr := refreshAppsCheck(info)
 	if checkerErr == nil {
 		return nil
 	}
 
-	// Get pending refresh information from compatible errors or synthesize a new one.
-	var refreshInfo *userclient.PendingSnapRefreshInfo
-	if err, ok := checkerErr.(*BusySnapError); ok {
-		refreshInfo = err.PendingSnapRefreshInfo()
-	} else {
-		refreshInfo = &userclient.PendingSnapRefreshInfo{
-			InstanceName: info.InstanceName(),
-		}
+	// carries the remaining inhibition time along with the BusySnapError
+	busyErr := &timedBusySnapError{}
+
+	// if it's not a snap busy error or the refresh is manual, surface the error
+	// to the user instead of notifying or delaying the refresh
+	if !snapsup.IsAutoRefresh || !errors.As(checkerErr, &busyErr.err) {
+		return checkerErr
 	}
 
 	// Decide on what to do depending on the state of the snap and the remaining
@@ -625,25 +762,26 @@ func inhibitRefresh(st *state.State, snapst *SnapState, info *snap.Info, checker
 		// time in the snap state's RefreshInhibitedTime field. This field is
 		// reset to nil on successful refresh.
 		snapst.RefreshInhibitedTime = &now
-		refreshInfo.TimeRemaining = (maxInhibition - now.Sub(*snapst.RefreshInhibitedTime)).Truncate(time.Second)
+		busyErr.timeRemaining = (maxInhibition - now.Sub(*snapst.RefreshInhibitedTime)).Truncate(time.Second)
 		Set(st, info.InstanceName(), snapst)
 	case now.Sub(*snapst.RefreshInhibitedTime) < maxInhibition:
 		// If we are still in the allowed window then just return the error but
 		// don't change the snap state again.
 		// TODO: as time left shrinks, send additional notifications with
 		// increasing frequency, allowing the user to understand the urgency.
-		refreshInfo.TimeRemaining = (maxInhibition - now.Sub(*snapst.RefreshInhibitedTime)).Truncate(time.Second)
+		busyErr.timeRemaining = (maxInhibition - now.Sub(*snapst.RefreshInhibitedTime)).Truncate(time.Second)
 	default:
-		// If we run out of time then consume the error that would normally
-		// inhibit refresh and notify the user that the snap is refreshing right
-		// now, by not setting the TimeRemaining field of the refresh
-		// notification message.
-		checkerErr = nil
+		// if the refresh inhibition window has ended, notify the user that the
+		// refresh is happening now and ignore the error
+		refreshInfo := busyErr.PendingSnapRefreshInfo()
+		asyncPendingRefreshNotification(context.TODO(), userclient.New(), refreshInfo)
+		// important to return "nil" type here instead of
+		// setting busyErr to nil as otherwise we return a nil
+		// interface which is not the nil type
+		return nil
 	}
 
-	// Send the notification asynchronously to avoid holding the state lock.
-	asyncPendingRefreshNotification(context.TODO(), userclient.New(), refreshInfo)
-	return checkerErr
+	return busyErr
 }
 
 // for testing outside of snapstate
