@@ -1417,8 +1417,9 @@ func (s *autorefreshGatingSuite) TestAutorefreshPhase1FeatureFlag(c *C) {
 	mockInstalledSnap(c, s.state, snapAyaml, useHook)
 
 	// gate-auto-refresh-hook feature not enabled, expect old-style refresh.
-	_, tss, err := snapstate.AutoRefresh(context.TODO(), st)
+	_, updateTss, err := snapstate.AutoRefresh(context.TODO(), st, nil)
 	c.Check(err, IsNil)
+	tss := updateTss.Refresh
 	c.Assert(tss, HasLen, 2)
 	c.Check(tss[0].Tasks()[0].Kind(), Equals, "prerequisites")
 	c.Check(tss[0].Tasks()[1].Kind(), Equals, "download-snap")
@@ -1429,8 +1430,9 @@ func (s *autorefreshGatingSuite) TestAutorefreshPhase1FeatureFlag(c *C) {
 	tr.Set("core", "experimental.gate-auto-refresh-hook", true)
 	tr.Commit()
 
-	_, tss, err = snapstate.AutoRefresh(context.TODO(), st)
+	_, updateTss, err = snapstate.AutoRefresh(context.TODO(), st, nil)
 	c.Check(err, IsNil)
+	tss = updateTss.Refresh
 	c.Assert(tss, HasLen, 2)
 	task := tss[0].Tasks()[0]
 	c.Check(task.Kind(), Equals, "conditional-auto-refresh")
@@ -2963,6 +2965,76 @@ func (s *autorefreshGatingSuite) TestSnapsNotHeldForeverBySnaps(c *C) {
 	c.Check(gatedSnaps, HasLen, 0)
 }
 
+func (s *autorefreshGatingSuite) TestLongestGatingHold(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	now := time.Now()
+	restore := snapstate.MockTimeNow(func() time.Time {
+		return now
+	})
+	defer restore()
+
+	mockInstalledSnap(c, st, snapAyaml, false)
+	mockInstalledSnap(c, st, snapCyaml, false)
+
+	_, err := snapstate.HoldRefresh(st, snapstate.HoldGeneral, "snap-a", 7*24*time.Hour, "snap-a")
+	c.Assert(err, IsNil)
+
+	_, err = snapstate.HoldRefresh(st, snapstate.HoldGeneral, "snap-c", 24*time.Hour, "snap-a")
+	c.Assert(err, IsNil)
+
+	holdTime, err := snapstate.LongestGatingHold(st, "snap-a")
+	c.Assert(err, IsNil)
+	c.Assert(holdTime.Equal(now.Add(7*24*time.Hour)), Equals, true)
+}
+
+func (s *autorefreshGatingSuite) TestGatingHoldEmptyTimeOnHoldNotFound(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	mockInstalledSnap(c, st, snapAyaml, false)
+
+	holdTime, err := snapstate.LongestGatingHold(st, "snap-a")
+	c.Assert(err, IsNil)
+	c.Assert(holdTime.IsZero(), Equals, true)
+}
+
+func (s *autorefreshGatingSuite) TestSystemHold(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	now := time.Now()
+	restore := snapstate.MockTimeNow(func() time.Time {
+		return now
+	})
+	defer restore()
+
+	mockInstalledSnap(c, st, snapAyaml, false)
+
+	err := snapstate.HoldRefreshesBySystem(st, snapstate.HoldGeneral, "forever", []string{"snap-a"})
+	c.Assert(err, IsNil)
+
+	holdTime, err := snapstate.SystemHold(st, "snap-a")
+	c.Assert(err, IsNil)
+	c.Assert(holdTime.Equal(now.Add(snapstate.MaxDuration)), Equals, true)
+}
+
+func (s *autorefreshGatingSuite) TestSystemHoldEmptyTimeOnHoldNotFound(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	mockInstalledSnap(c, st, snapAyaml, false)
+
+	holdTime, err := snapstate.SystemHold(st, "snap-a")
+	c.Assert(err, IsNil)
+	c.Assert(holdTime.IsZero(), Equals, true)
+}
+
 func verifyPhasedAutorefreshTasks(c *C, tasks []*state.Task, expected []string) {
 	c.Assert(len(tasks), Equals, len(expected))
 	for i, t := range tasks {
