@@ -170,6 +170,8 @@ type fakeStore struct {
 	downloadError   map[string]error
 	state           *state.State
 	seenPrivacyKeys map[string]bool
+
+	downloadCallback func()
 }
 
 func (f *fakeStore) pokeStateLock() {
@@ -241,6 +243,8 @@ func (f *fakeStore) snap(spec snapSpec) (*snap.Info, error) {
 		confinement = "classic"
 	case "some-epoch-snap":
 		epoch = snap.E("42")
+	case "firmware-updater":
+		snapID = "EI0D1KHjP8XiwMZKqSjuh6W8zvcowUVP"
 	case "snapd-desktop-integration":
 		snapID = "IrwRHakqtzhFRHJOOPxKVPU0Kk7Erhcu"
 	}
@@ -695,6 +699,10 @@ func (f *fakeStore) Download(ctx context.Context, name, targetFn string, snapInf
 	if _, key := snap.SplitInstanceName(name); key != "" {
 		return fmt.Errorf("internal error: unsupported download with instance name %q", name)
 	}
+	if f.downloadCallback != nil {
+		f.downloadCallback()
+	}
+
 	var macaroon string
 	if user != nil {
 		macaroon = user.StoreMacaroon
@@ -711,8 +719,10 @@ func (f *fakeStore) Download(ctx context.Context, name, targetFn string, snapInf
 	})
 	f.fakeBackend.appendOp(&fakeOp{op: "storesvc-download", name: name})
 
-	pb.SetTotal(float64(f.fakeTotalProgress))
-	pb.Set(float64(f.fakeCurrentProgress))
+	if pb != nil {
+		pb.SetTotal(float64(f.fakeTotalProgress))
+		pb.Set(float64(f.fakeCurrentProgress))
+	}
 
 	if e, ok := f.downloadError[name]; ok {
 		return e
@@ -830,8 +840,9 @@ func (f *fakeSnappyBackend) OpenSnapFile(snapFilePath string, si *snap.SideInfo)
 }
 
 // XXX: this is now something that is overridden by tests that need a
-//      different service setup so it should be configurable and part
-//      of the fakeSnappyBackend?
+//
+//	different service setup so it should be configurable and part
+//	of the fakeSnappyBackend?
 var servicesSnapYaml = `name: services-snap
 apps:
   svc1:
@@ -1247,6 +1258,13 @@ func (f *fakeSnappyBackend) DiscardSnapNamespace(snapName string) error {
 	return f.maybeErrForLastOp()
 }
 
+func (f *fakeSnappyBackend) RemoveAllSnapAppArmorProfiles() error {
+	f.appendOp(&fakeOp{
+		op: "remove-apparmor-profiles",
+	})
+	return f.maybeErrForLastOp()
+}
+
 func (f *fakeSnappyBackend) RemoveSnapInhibitLock(snapName string) error {
 	f.appendOp(&fakeOp{
 		op:   "remove-inhibit-lock",
