@@ -69,13 +69,13 @@ hooks:
 	s.state = state.New(nil)
 }
 
-func (s *refreshSuite) TestSoftNothingRunningRefreshCheck(c *C) {
+func (s *refreshSuite) TestRefreshCheck(c *C) {
 	// Services are not blocking soft refresh check,
 	// they will be stopped before refresh.
 	s.pids = map[string][]int{
 		"snap.pkg.daemon": {100},
 	}
-	err := snapstate.SoftNothingRunningRefreshCheck(s.info)
+	err := snapstate.RefreshCheck(s.info)
 	c.Check(err, IsNil)
 
 	// Apps are blocking soft refresh check. They are not stopped by
@@ -85,14 +85,14 @@ func (s *refreshSuite) TestSoftNothingRunningRefreshCheck(c *C) {
 		"snap.pkg.daemon": {100},
 		"snap.pkg.app":    {101},
 	}
-	err = snapstate.SoftNothingRunningRefreshCheck(s.info)
+	err = snapstate.RefreshCheck(s.info)
 	c.Assert(err, NotNil)
 	c.Check(err.Error(), Equals, `snap "pkg" has running apps (app), pids: 101`)
 	c.Check(err.(*snapstate.BusySnapError).Pids(), DeepEquals, []int{101})
 
 	// Unless refresh-mode is set to "ignore-running"
 	s.info.Apps["app"].RefreshMode = "ignore-running"
-	err = snapstate.SoftNothingRunningRefreshCheck(s.info)
+	err = snapstate.RefreshCheck(s.info)
 	c.Assert(err, IsNil)
 	s.info.Apps["app"].RefreshMode = ""
 
@@ -101,7 +101,7 @@ func (s *refreshSuite) TestSoftNothingRunningRefreshCheck(c *C) {
 	s.pids = map[string][]int{
 		"snap.pkg.hook.configure": {105},
 	}
-	err = snapstate.SoftNothingRunningRefreshCheck(s.info)
+	err = snapstate.RefreshCheck(s.info)
 	c.Assert(err, NotNil)
 	c.Check(err.Error(), Equals, `snap "pkg" has running hooks (configure), pids: 105`)
 	c.Check(err.(*snapstate.BusySnapError).Pids(), DeepEquals, []int{105})
@@ -111,53 +111,12 @@ func (s *refreshSuite) TestSoftNothingRunningRefreshCheck(c *C) {
 		"snap.pkg.hook.configure": {105},
 		"snap.pkg.app":            {106},
 	}
-	err = snapstate.SoftNothingRunningRefreshCheck(s.info)
+	err = snapstate.RefreshCheck(s.info)
 	c.Assert(err, NotNil)
 	c.Check(err.Error(), Equals, `snap "pkg" has running apps (app) and hooks (configure), pids: 105,106`)
 	c.Check(err.(*snapstate.BusySnapError).Pids(), DeepEquals, []int{105, 106})
 }
 
-func (s *refreshSuite) TestHardNothingRunningRefreshCheck(c *C) {
-	// Services are ignored by hard refresh check.
-	s.pids = map[string][]int{
-		"snap.pkg.daemon": {100},
-	}
-	err := snapstate.HardNothingRunningRefreshCheck(s.info)
-	c.Assert(err, IsNil)
-
-	// When the service is supposed to endure refreshes it will not be
-	// stopped. As such such services cannot block refresh.
-	s.info.Apps["daemon"].RefreshMode = "endure"
-	err = snapstate.HardNothingRunningRefreshCheck(s.info)
-	c.Check(err, IsNil)
-	s.info.Apps["daemon"].RefreshMode = ""
-
-	// Applications are also blocking hard refresh check.
-	s.pids = map[string][]int{
-		"snap.pkg.app": {101},
-	}
-	err = snapstate.HardNothingRunningRefreshCheck(s.info)
-	c.Assert(err, NotNil)
-	c.Check(err.Error(), Equals, `snap "pkg" has running apps (app), pids: 101`)
-	c.Assert(err, FitsTypeOf, &snapstate.BusySnapError{})
-	c.Check(err.(*snapstate.BusySnapError).Pids(), DeepEquals, []int{101})
-
-	// Unless refresh-mode is set to "ignore-running"
-	s.info.Apps["app"].RefreshMode = "ignore-running"
-	err = snapstate.HardNothingRunningRefreshCheck(s.info)
-	c.Assert(err, IsNil)
-	s.info.Apps["app"].RefreshMode = ""
-
-	// Hooks are equally blocking hard refresh check.
-	s.pids = map[string][]int{
-		"snap.pkg.hook.configure": {105},
-	}
-	err = snapstate.HardNothingRunningRefreshCheck(s.info)
-	c.Assert(err, NotNil)
-	c.Check(err.Error(), Equals, `snap "pkg" has running hooks (configure), pids: 105`)
-	c.Check(err.(*snapstate.BusySnapError).Pids(), DeepEquals, []int{105})
-}
-
 func (s *refreshSuite) TestPendingSnapRefreshInfo(c *C) {
 	err := snapstate.NewBusySnapError(s.info, nil, nil, nil)
 	refreshInfo := err.PendingSnapRefreshInfo()
@@ -202,15 +161,16 @@ func (s *refreshSuite) TestDoSoftRefreshCheckAllowed(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 	snapst, info := s.addFakeInstalledSnap()
+	snapsup := &snapstate.SnapSetup{Flags: snapstate.Flags{IsAutoRefresh: true}}
 
 	// Pretend that snaps can refresh normally.
-	restore := snapstate.MockGenericRefreshCheck(func(info *snap.Info, canAppRunDuringRefresh func(app *snap.AppInfo) bool) error {
+	restore := snapstate.MockRefreshAppsCheck(func(info *snap.Info) error {
 		return nil
 	})
 	defer restore()
 
 	// Soft refresh should not fail.
-	err := snapstate.SoftCheckNothingRunningForRefresh(s.state, snapst, info)
+	err := snapstate.SoftCheckNothingRunningForRefresh(s.state, snapst, snapsup, info)
 	c.Assert(err, IsNil)
 
 	// In addition, the inhibition lock is not set.
@@ -224,15 +184,16 @@ func (s *refreshSuite) TestDoSoftRefreshCheckDisallowed(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 	snapst, info := s.addFakeInstalledSnap()
+	snapsup := &snapstate.SnapSetup{Flags: snapstate.Flags{IsAutoRefresh: true}}
 
 	// Pretend that snaps cannot refresh.
-	restore := snapstate.MockGenericRefreshCheck(func(info *snap.Info, canAppRunDuringRefresh func(app *snap.AppInfo) bool) error {
+	restore := snapstate.MockRefreshAppsCheck(func(info *snap.Info) error {
 		return snapstate.NewBusySnapError(info, []int{123}, nil, nil)
 	})
 	defer restore()
 
 	// Soft refresh should fail with a proper error.
-	err := snapstate.SoftCheckNothingRunningForRefresh(s.state, snapst, info)
+	err := snapstate.SoftCheckNothingRunningForRefresh(s.state, snapst, snapsup, info)
 	c.Assert(err, ErrorMatches, `snap "pkg" has running apps or hooks, pids: 123`)
 
 	// Validity check: the inhibition lock was not set.
@@ -247,15 +208,16 @@ func (s *refreshSuite) TestDoHardRefreshFlowRefreshAllowed(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 	snapst, info := s.addFakeInstalledSnap()
+	snapsup := &snapstate.SnapSetup{Flags: snapstate.Flags{IsAutoRefresh: true}}
 
 	// Pretend that snaps can refresh normally.
-	restore := snapstate.MockGenericRefreshCheck(func(info *snap.Info, canAppRunDuringRefresh func(app *snap.AppInfo) bool) error {
+	restore := snapstate.MockRefreshAppsCheck(func(info *snap.Info) error {
 		return nil
 	})
 	defer restore()
 
 	// Hard refresh should not fail and return a valid lock.
-	lock, err := snapstate.HardEnsureNothingRunningDuringRefresh(backend, s.state, snapst, info)
+	lock, err := snapstate.HardEnsureNothingRunningDuringRefresh(backend, s.state, snapst, snapsup, info)
 	c.Assert(err, IsNil)
 	c.Assert(lock, NotNil)
 	defer lock.Close()
@@ -275,15 +237,16 @@ func (s *refreshSuite) TestDoHardRefreshFlowRefreshDisallowed(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
 	snapst, info := s.addFakeInstalledSnap()
+	snapsup := &snapstate.SnapSetup{Flags: snapstate.Flags{IsAutoRefresh: true}}
 
 	// Pretend that snaps cannot refresh.
-	restore := snapstate.MockGenericRefreshCheck(func(info *snap.Info, canAppRunDuringRefresh func(app *snap.AppInfo) bool) error {
+	restore := snapstate.MockRefreshAppsCheck(func(info *snap.Info) error {
 		return snapstate.NewBusySnapError(info, []int{123}, nil, nil)
 	})
 	defer restore()
 
 	// Hard refresh should fail and not return a lock.
-	lock, err := snapstate.HardEnsureNothingRunningDuringRefresh(backend, s.state, snapst, info)
+	lock, err := snapstate.HardEnsureNothingRunningDuringRefresh(backend, s.state, snapst, snapsup, info)
 	c.Assert(err, ErrorMatches, `snap "pkg" has running apps or hooks, pids: 123`)
 	c.Assert(lock, IsNil)
 
