@@ -114,6 +114,10 @@ type Config struct {
 
 	// Proxy returns the HTTP proxy to use when talking to the store
 	Proxy func(*http.Request) (*url.URL, error)
+
+	// AssertionMaxFormats if set provides a way to override
+	// the assertion max formats sent to the store as supported.
+	AssertionMaxFormats map[string]int
 }
 
 // setBaseURL updates the store API's base URL in the Config. Must not be used
@@ -343,6 +347,14 @@ type sectionResults struct {
 	} `json:"_embedded"`
 }
 
+type CategoryDetails struct {
+	Name string `json:"name"`
+}
+
+type categoryResults struct {
+	Categories []CategoryDetails `json:"categories"`
+}
+
 // The default delta format if not configured.
 var defaultSupportedDeltaFormat = "xdelta3"
 
@@ -419,6 +431,12 @@ func New(cfg *Config, dauthCtx DeviceAndAuthContext) *Store {
 	return store
 }
 
+// SetAssertionMaxFormats allows to change the assertion max formats to send
+// for a store already in use.
+func (s *Store) SetAssertionMaxFormats(maxFormats map[string]int) {
+	s.cfg.AssertionMaxFormats = maxFormats
+}
+
 // API endpoint paths
 const (
 	// see https://dashboard.snapcraft.io/docs/
@@ -438,6 +456,7 @@ const (
 	snapInfoEndpPath   = "v2/snaps/info"
 	cohortsEndpPath    = "v2/cohorts"
 	findEndpPath       = "v2/snaps/find"
+	categoriesEndpPath = "v2/snaps/categories"
 
 	deviceNonceEndpPath   = "api/v1/snaps/auth/nonces"
 	deviceSessionEndpPath = "api/v1/snaps/auth/sessions"
@@ -553,7 +572,6 @@ var (
 type deviceAuthNeed int
 
 const (
-	//nolint:deadcode
 	deviceAuthPreferred deviceAuthNeed = iota
 	deviceAuthCustomStoreOnly
 )
@@ -832,26 +850,26 @@ func (s *Store) extractSuggestedCurrency(resp *http.Response) {
 
 // ordersResult encapsulates the order data sent to us from the software center agent.
 //
-// {
-//   "orders": [
-//     {
-//       "snap_id": "abcd1234efgh5678ijkl9012",
-//       "currency": "USD",
-//       "amount": "2.99",
-//       "state": "Complete",
-//       "refundable_until": null,
-//       "purchase_date": "2016-09-20T15:00:00+00:00"
-//     },
-//     {
-//       "snap_id": "abcd1234efgh5678ijkl9012",
-//       "currency": null,
-//       "amount": null,
-//       "state": "Complete",
-//       "refundable_until": null,
-//       "purchase_date": "2016-09-20T15:00:00+00:00"
-//     }
-//   ]
-// }
+//	{
+//	  "orders": [
+//	    {
+//	      "snap_id": "abcd1234efgh5678ijkl9012",
+//	      "currency": "USD",
+//	      "amount": "2.99",
+//	      "state": "Complete",
+//	      "refundable_until": null,
+//	      "purchase_date": "2016-09-20T15:00:00+00:00"
+//	    },
+//	    {
+//	      "snap_id": "abcd1234efgh5678ijkl9012",
+//	      "currency": null,
+//	      "amount": null,
+//	      "state": "Complete",
+//	      "refundable_until": null,
+//	      "purchase_date": "2016-09-20T15:00:00+00:00"
+//	    }
+//	  ]
+//	}
 type ordersResult struct {
 	Orders []*order `json:"orders"`
 }
@@ -1245,6 +1263,32 @@ func (s *Store) Sections(ctx context.Context, user *auth.UserState) ([]string, e
 	return sectionNames, nil
 }
 
+// Categories retrieves the list of available store categories.
+func (s *Store) Categories(ctx context.Context, user *auth.UserState) ([]CategoryDetails, error) {
+	reqOptions := &requestOptions{
+		Method:   "GET",
+		URL:      s.endpointURL(categoriesEndpPath, nil),
+		Accept:   jsonContentType,
+		APILevel: apiV2Endps,
+	}
+
+	var categoryData categoryResults
+	resp, err := s.retryRequestDecodeJSON(context.TODO(), reqOptions, user, &categoryData, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	if resp.StatusCode != 200 {
+		return nil, respToError(resp, "retrieve categories")
+	}
+
+	if ct := resp.Header.Get("Content-Type"); ct != jsonContentType {
+		return nil, fmt.Errorf("received an unexpected content type (%q) when trying to retrieve the categories via %q", ct, resp.Request.URL)
+	}
+
+	return categoryData.Categories, nil
+}
+
 // WriteCatalogs queries the "commands" endpoint and writes the
 // command names into the given io.Writer.
 func (s *Store) WriteCatalogs(ctx context.Context, names io.Writer, adder SnapAdder) error {
