@@ -31,33 +31,23 @@ remap_one() {
 }
 
 cmd_install() {
-    set -x
     # shellcheck disable=SC2068
     zypper install -y $@
-    set +x
 }
 
 cmd_is_installed() {
-    set -x
     rpm -qi "$1" >/dev/null 2>&1
-    set +x
 }
 
 cmd_query() {
-    set -x
     zypper info "$1"
-    set +x
 }
 
 cmd_list_installed() {
-    set -x
     rpm -qa | sort
-    set +x
 }
 
 cmd_remove() {
-    set -x
     # shellcheck disable=SC2068
     zypper remove -y $@
-    set +x
 }
